@@ -1086,7 +1086,7 @@ sub apply_row_methods {
 		inlineNavLink => sub {
 			my $self = shift;
 			my $text = shift || '<span>open</span>';
-			my %attrs = ( class => "ra-icon-magnify-tiny", @_ );
+			my %attrs = ( class => "ra-nav-link ra-icon-magnify-tiny", @_ );
 
 			my $title = $self->getDisplayValue or return undef;
 			my $url = $self->getRestPath or return undef;
