@@ -895,7 +895,7 @@ Ext.ux.RapidApp.DataStoreAppField = Ext.extend(Ext.ux.RapidApp.ClickActionField,
 		// -- Automatically hide the window if it is visible and a nav/load target
 		// event happens in the main loadTarget. This can happen if, for example,
 		// the user clicks an 'open' link within the grid combo to a related object
-		var loadTarget = Ext.getCmp("explorer-id").getComponent("load-target");
+		var loadTarget = Ext.getCmp("main-load-target");
 		if(loadTarget){
 			loadTarget.on('navload',function(){
 				if(this.appWindow && this.appWindow.isVisible()){
